/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.runtime.CorrelationSetInitialization;
import org.ow2.orchestra.facade.runtime.impl.CorrelationSetInitializationImpl;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.util.XmlConstants;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 *
 * @author Gael Lalire
 */
public class CorrelationSetInitializationFullImpl implements CorrelationSetInitialization {

  private static final long serialVersionUID = 4978973329065042473L;

  // for persistence
  protected long dbid;

  private ActivityInstanceUUID activityUUID;

  private Date date;

  private String name;

  protected Map<QName, Document> propertyValues;

  protected CorrelationSetInitializationFullImpl() {
  }

  public CorrelationSetInitializationFullImpl(final CorrelationSetInitialization record) {
    this.activityUUID = record.getActivityUUID();
    this.date = new Date(record.getDate().getTime());
    this.name = record.getName();
    this.propertyValues = new HashMap<QName, Document>();
    for (final Entry<QName, Node> entry : record.getPropertyValues().entrySet()) {
      final Element el = XmlUtil.getDocumentWithOneElement(
          new QName(XmlConstants.ELT_DEFAULT_NS, XmlConstants.ELT_DEFAULT_ELEMENT));
      el.appendChild(el.getOwnerDocument().importNode(entry.getValue(), true));
      this.propertyValues.put(entry.getKey(), el.getOwnerDocument());
    }
  }

  public CorrelationSetInitializationFullImpl(final ActivityInstanceUUID activityUUID, final Date date, final String name,
    final Map<QName, Document> map) {
    this.activityUUID = activityUUID;
    this.date = new Date(date.getTime());
    this.name = name;
    this.propertyValues = new HashMap<QName, Document>();
    for (final Entry<QName, Document> entry : map.entrySet()) {
      this.propertyValues.put(entry.getKey(), XmlUtil.getDocumentFromString(XmlUtil.toString(entry.getValue())));
    }
  }

  public ActivityInstanceUUID getActivityUUID() {
    return this.activityUUID;
  }

  public Date getDate() {
    return this.date;
  }

  public String getName() {
    return this.name;
  }

  /**
   * @return the propertyValues
   */
  public Map<QName, Node> getPropertyValues() {
    final Map<QName, Node> res = new HashMap<QName, Node>();
    for (final Entry<QName, Document> prop : this.propertyValues.entrySet()) {
      final Document doc = prop.getValue();
      if (doc != null) {
        final Element elt = doc.getDocumentElement();
        final Element child = XmlUtil.element(elt);
        if (child != null) {
          res.put(prop.getKey(), child);
        } else {
          res.put(prop.getKey(), elt.getFirstChild());
        }
      }
    }
    return res;
  }

  public CorrelationSetInitialization copy() {
    return new CorrelationSetInitializationImpl(this);
  }

}
