/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import org.ow2.orchestra.definition.activity.Exit;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.EnvTool;

public final class ExitInstanceCommand implements Command<InstanceNotFoundException> {

  /**
   *
   */
  private final ProcessInstanceUUID processInstanceUUID;
  private static final long serialVersionUID = 1L;

  /**
   * @param processInstanceUUID
   */
  public ExitInstanceCommand(final ProcessInstanceUUID processInstanceUUID) {
    this.processInstanceUUID = processInstanceUUID;
  }
  public InstanceNotFoundException execute(final Environment environment) {
    final BpelExecution instance = EnvTool.getRepository().getInstance(this.processInstanceUUID);
    if (instance == null) {
      return new InstanceNotFoundException(this.processInstanceUUID);
    }
    if (instance.getProcessInstanceState().equals(ActivityState.RUNNING)
        || instance.getProcessInstanceState().equals(ActivityState.SUSPENDED)) {
      new Exit().executeActivity(instance);
    }

    return null;
  }
}
