/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import java.util.HashSet;
import java.util.Set;

import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.services.Querier;
import org.ow2.orchestra.util.EnvTool;

public final class FindActivityDefinitionsCommand implements Command<Set<ActivityDefinition>> {

  private static final long serialVersionUID = 3912214314959328185L;

  private final ProcessDefinitionUUID processUUID;


  public FindActivityDefinitionsCommand(final ProcessDefinitionUUID processUUID) {
    this.processUUID = processUUID;
  }

  public Set<ActivityDefinition> execute(final Environment environment) {
    final Set<ActivityDefinition> result = new HashSet<ActivityDefinition>();
    final Querier querier = EnvTool.getQuerier();
    if (querier.getProcessDefinition(this.processUUID) == null) {
      throw new OrchestraWrapperException(new ProcessNotFoundException(this.processUUID));
    }
    final Set<ActivityFullDefinition> activityFullDefinitions = querier.findActivityDefinitions(this.processUUID);
    for (final ActivityFullDefinition activityFullDefinition : activityFullDefinitions) {
      result.add(activityFullDefinition.copy());
    }
    return result;
  }
}
