/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.EnvTool;

public class GetInstanceCommand implements Command<ProcessInstanceData> {

  private static final long serialVersionUID = -4282818977685054488L;

  private final ProcessInstanceUUID processUUID;

  public GetInstanceCommand(final ProcessInstanceUUID processUUID) {
    this.processUUID = processUUID;
  }

  public ProcessInstanceData execute(final Environment environment) {
    final BpelExecution bpelInstance = EnvTool.getRepository().getInstance(this.processUUID);
    if (bpelInstance != null) {
      return bpelInstance.getProcessInstanceData();
    }
    throw new OrchestraWrapperException(new InstanceNotFoundException(this.processUUID));
  }
}
