/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import org.ow2.orchestra.facade.data.runtime.JobData;
import org.ow2.orchestra.pvm.PvmException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.env.Transaction;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.pvm.internal.job.JobImpl;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobAddedNotification;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobExecutor;
import org.ow2.orchestra.util.EnvTool;

/**
 * @author Guillaume Porcher
 *
 */
public final class RetryJobCommand implements Command<Boolean> {

  private static final long serialVersionUID = 3748547428936058760L;

  private final int retries;

  private final JobData jobData;

  /**
   * @param retries
   * @param jobData
   */
  public RetryJobCommand(final int retries, final JobData jobData) {
    this.retries = retries;
    this.jobData = jobData;
  }
  public Boolean execute(final Environment environment) {
    // Get the job
    final JobImpl< ? > job = EnvTool.getRepository().getJob(this.jobData.getJobId());
    if (job == null) {
      // if job not found, return false
      return Boolean.FALSE;
    }
    // Job found, update retries
    job.setRetries(this.retries);
    // notify the jobExecutor at the end of the transaction
    final JobExecutor jobExecutor = environment.get(JobExecutor.class);
    if (jobExecutor != null) {
      final Transaction transaction = environment.get(Transaction.class);
      if (transaction == null) {
        throw new PvmException("no transaction in environment");
      }
      final JobAddedNotification jobNotificator = new JobAddedNotification(
          jobExecutor);
      transaction.registerSynchronization(jobNotificator);
    }
    return Boolean.TRUE;
  }
}
