/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.lang.function;

import java.util.List;

import org.jaxen.Context;
import org.jaxen.Function;
import org.jaxen.FunctionCallException;
import org.w3c.dom.Node;
import org.w3c.dom.Text;


/**
 * @author Guillaume Porcher
 *
 * XPath function to get a boolean object from an xml string.
 * Returns <code>true</code> if the string parameter is "true" or "1", or <code>false</code> otherwise.
 *
 */
public class BooleanFromStringFunction implements Function {

  public static final String FUNCTION_NAME = "boolean-from-string";

  public Object call(final Context context, final List args) throws FunctionCallException {
    if (args.size() != 1) {
      throw new FunctionCallException(
          "Wrong number of argument for the orchestra:boolean-from-string(string) function");
    }
    // get arg0
    final Object node = args.get(0);
    String value = null;
    // node can be a Text, a String or a list of nodes containing only one (text) node.
    if (node instanceof Text) {
      value = ((Text)node).getTextContent();
    } else if (node instanceof List) {
      final List<Node> nodeList = (List) node;
      if (nodeList.get(0) instanceof Text) {
        value = ((Text)nodeList.get(0)).getTextContent();
      }
    } else if (node instanceof String) {
      value = (String) node;
    }

    // If invalid type, throw exception
    if (value == null) {
      throw new FunctionCallException("invalid type for argument " + node + ". A string is expected.");
    }

    // return boolean value of the string
    if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("1")) {
      return Boolean.TRUE;
    }
    return Boolean.FALSE;
  }

}
