package org.ow2.orchestra.persistence;

import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.deployment.ProcessDeploymentCounter;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.pvm.session.PvmDbSession;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.WaitingExecution;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.services.ResultIterator;
import org.w3c.dom.Document;

public interface RuntimeDbSession extends PvmDbSession {

  /** searches for the latest version of the given BpelProcess */
  BpelProcess findActiveBpelProcess(QName processQName);

  /** searches for the all deployed versions of the given BpelProcess */
  List<BpelProcess> findBpelProcesses(QName processQName, boolean lock);

  BpelProcess findBpelProcess(ProcessDefinitionUUID processUUID, boolean lock);

  /** searches for the latest version of the given BpelProcess */
  BpelProcess getBpelProcess(ProcessDefinitionUUID pduuid);

  /** searches for all the BpelProcesses in the database */
  List<BpelProcess> getBpelProcesses();

  /** searches for all the BpelInstances in the database */
  List<BpelExecution> getBpelInstances();

  /** searches for all the BpelInstances in the database */
  BpelExecution getBpelInstance(ProcessInstanceUUID processInstanceUUID);

  /**
   * searches for all the Instances instances of the given BpelProcess in the
   * database
   */
  List<BpelExecution> getBpelInstances(QName processQName);
  List<BpelExecution> getBpelInstances(ProcessDefinitionUUID processUUID);

  /**
   * @return
   */
  ProcessDeploymentCounter getProcessDeploymentCounter();

  ResultIterator<WaitingExecution> getWaitingExecutions(
      ReceivingElement receivingElement,
      Map<QName, Document> csProperties);
  WaitingExecution getFirstWaitingExecution(InboundMessageElement inboundMessageElement);

  ResultIterator<PendingMessage> getPendingMessages(OperationKey operationKey);

  void clearProcessCache();

}
