/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.impl;

import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.deployment.ProcessDeploymentCounter;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.RuntimeDbSession;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.job.JobImpl;
import org.ow2.orchestra.pvm.job.Job;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.WaitingExecution;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.services.ResultIterator;
import org.ow2.orchestra.services.itf.Repository;
import org.w3c.dom.Document;

public class DbRepository implements Repository {

  /*
   * PROCESS REPOSITORY
   */
  public long getNextProcessCount() {
    ProcessDeploymentCounter processDeploymentCounter = this.getBpelPersistenceService().getProcessDeploymentCounter();
    if (processDeploymentCounter == null) {
      processDeploymentCounter = new ProcessDeploymentCounter();
      this.getBpelPersistenceService().save(processDeploymentCounter);
    }
    return processDeploymentCounter.getAndIncrementProcessCounter();
  }

  public BpelProcess getActiveProcess(final QName processQName) {
    return this.getBpelPersistenceService().findActiveBpelProcess(processQName);
  }
  public BpelProcess getProcess(final ProcessDefinitionUUID processUUID) {
    return this.getBpelPersistenceService().getBpelProcess(processUUID);
  }

  public List<BpelProcess> getProcesses(final QName processQName) {
    return this.getBpelPersistenceService().findBpelProcesses(processQName, false);
  }

  public List<BpelProcess> getProcesses() {
    return this.getBpelPersistenceService().getBpelProcesses();
  }

  public BpelProcess removeProcess(final QName processQName) {
    final BpelProcess process = this.getActiveProcess(processQName);
    if (process != null) {
      this.getBpelPersistenceService().delete(process);
    }
    return process;
  }

  public BpelProcess removeProcess(final ProcessDefinitionUUID pduuid) {
    final BpelProcess process = this.getBpelPersistenceService().getBpelProcess(pduuid);
    if (process != null) {
      this.getBpelPersistenceService().delete(process);
    }
    return process;
  }

  public void storeProcess(final BpelProcess process) {
    this.getBpelPersistenceService().save(process);
  }

  public List<BpelProcess> lockProcesses(final QName processQName) {
    return this.getBpelPersistenceService().findBpelProcesses(processQName, true);
  }

  public void refreshProcess(final ProcessDefinitionUUID pduuid) {
    final BpelProcess process = this.getProcess(pduuid);
    if (process != null) {
      this.getBpelPersistenceService().clearProcessCache();
    }
  }

  /*
   * INSTANCE REPOSITORY
   */
  public BpelExecution getInstance(final ProcessInstanceUUID processInstanceUUID) {
    return this.getBpelPersistenceService().getBpelInstance(processInstanceUUID);
  }

  public List<BpelExecution> getInstances() {
    return this.getBpelPersistenceService().getBpelInstances();
  }

  public List<BpelExecution> getInstances(final QName processQName) {
    return this.getBpelPersistenceService().getBpelInstances(processQName);
  }

  public List<BpelExecution> getInstances(final ProcessDefinitionUUID processUUID) {
    return this.getBpelPersistenceService().getBpelInstances(processUUID);
  }
  public void removeInstance(final BpelExecution instance) {
    this.removeInstance(instance, false);
  }

  public void storeExecution(final Execution child) {
    this.getBpelPersistenceService().save(child);
  }
  public void removeExecution(final Execution child) {
    this.getBpelPersistenceService().delete(child);
  }

  public void removeInstance(final BpelExecution instance, final boolean force) {
    if (force) {
      if (instance != null
          && (instance.getProcessInstanceState().equals(ActivityState.RUNNING)
              || instance.getProcessInstanceState().equals(ActivityState.SUSPENDED))
      ) {
        instance.exit();
      }
    }
    if (instance != null) {
      this.getBpelPersistenceService().delete(instance.getProcessInstance());
    }
  }

  public void storeInstance(final BpelExecution instance) {
    this.getBpelPersistenceService().save(instance.getProcessInstance());
  }

  public BpelExecution getUpdatedExecution(final long executionId) {
    return this.getBpelPersistenceService().get(BpelExecution.class, Long.valueOf(executionId));
  }

  public List<Job> findJobsWithException() {
    return this.getBpelPersistenceService().findJobsWithException();
  }

  public JobImpl< ? > getJob(final long jobId) {
    return this.getBpelPersistenceService().get(JobImpl.class, Long.valueOf(jobId));
  }

  public ResultIterator<WaitingExecution> getWaitingExecutions(
      final ReceivingElement receivingElement,
      final Map<QName, Document> csProperties) {
    return this.getBpelPersistenceService().getWaitingExecutions(receivingElement, csProperties);
  }

  public WaitingExecution getFirstWaitingExecution(final InboundMessageElement inboundMessageElement) {
    return this.getBpelPersistenceService().getFirstWaitingExecution(inboundMessageElement);
  }

  public void storePendingMessage(final PendingMessage pendingMessage) {
    this.getBpelPersistenceService().save(pendingMessage);
  }

  public void removePendingMessage(final PendingMessage pendingMessage) {
    this.getBpelPersistenceService().delete(pendingMessage);
  }

  public ResultIterator<PendingMessage> getPendingMessages(final OperationKey operationKey) {
    return this.getBpelPersistenceService().getPendingMessages(operationKey);
  }
  /*
   * COMMON METHODS
   */
  private RuntimeDbSession getBpelPersistenceService() {
    return Environment.getCurrent().get(RuntimeDbSession.class);
  }

}
