/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.wsdl.Port;
import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class ProcessEndpoint {

  private static class ProcessVersionEndpoint {
    private final ProcessDefinitionUUID processUUID;
    private final Map<String, Boolean> isOneWay;
    private final Map<String, Boolean> locks;

    public ProcessVersionEndpoint(final ProcessDefinitionUUID processUUID, final Map<String, Boolean> isOneWay, final Map<String, Boolean> locks) {
      this.processUUID = processUUID;
      this.isOneWay = isOneWay;
      this.locks = locks;
    }
    public ProcessDefinitionUUID getProcessUUID() {
      return this.processUUID;
    }
    public Map<String, Boolean> getIsOneWay() {
      return this.isOneWay;
    }
    public Map<String, Boolean> getLocks() {
      return this.locks;
    }

  }

  private final QName processQName;
  private final Port port;
  private Map<String, Boolean> isOneWay = new HashMap<String, Boolean>();
  private Map<String, Boolean> locks = new HashMap<String, Boolean>();
  private final Map<ProcessDefinitionUUID, ProcessVersionEndpoint> versions =
    new HashMap<ProcessDefinitionUUID, ProcessVersionEndpoint>();

  public ProcessEndpoint(final QName processQName, final Port port) {
    this.processQName = processQName;
    this.port = port;
  }

  public boolean isLock(final String operationName) {
    return this.locks.get(operationName);
  }
  public boolean isOneWay(final String operationName) {
    return this.isOneWay.get(operationName);
  }
  public QName getProcessQName() {
    return this.processQName;
  }

  public void addProcessVersion(
      final ProcessDefinitionUUID processDefinitionUUID,
      final Map<String, Boolean> isOneWayMap,
      final Map<String, Boolean> locksMap) {
    this.versions.put(processDefinitionUUID, new ProcessVersionEndpoint(processDefinitionUUID, isOneWayMap, locksMap));
    this.updateDatas();
  }

  public void removeProcessVersion(final ProcessDefinitionUUID processDefinitionUUID) {
    this.versions.remove(processDefinitionUUID);
    this.updateDatas();
  }

  public boolean isUndeployed() {
    return this.versions.isEmpty();
  }

  private void updateDatas() {
    final Map<String, Boolean> newOneWay = new HashMap<String, Boolean>();
    final Map<String, Boolean> newLocks = new HashMap<String, Boolean>();

    for (final ProcessVersionEndpoint version : this.versions.values()) {
      for (final Entry<String, Boolean> e : version.getIsOneWay().entrySet()) {
        newOneWay.put(e.getKey(), e.getValue());
      }
      for (final Entry<String, Boolean> e : version.getLocks().entrySet()) {
        if (newLocks.containsKey(e.getKey())) {
          newLocks.put(e.getKey(), e.getValue() || newLocks.get(e.getKey()));
        } else {
          newLocks.put(e.getKey(), e.getValue());
        }
      }
    }
    this.isOneWay = newOneWay;
    this.locks = newLocks;
  }


  /**
   * @return the port
   */
  public Port getPort() {
    return this.port;
  }
}